#ifndef __COMMANDCLIENT_H__
#define __COMMANDCLIENT_H__

#include <string>

namespace Aspera {

namespace Command {

    class CmdReply;
    class CmdReplyInfo;
    class CmdReplyFile;
    class CmdReplyMd5sum;
    class CmdReplyMounts;
    class CmdReplySize;
    class CmdError;

   /*! \brief
    *  The class that can issue commands and return the results
    */
    class CommandClient
    {
    public:
        static CommandClient* CreateInstance();

       /*! \brief
        *  Connects to the specified remote computer using a Putty generated private key file for authentications and returns the remote system information
        */
        virtual CmdReplyInfo*   connect(char* host, char* user, char* passphrase, char* privkeyfile, int port, CmdError& cmderr) = 0;
       /*! \brief
        *  Connects to the specified remote computer using a username and password for authentication and returns the remote system information
        */
        virtual CmdReplyInfo*   connect(char* host, char* user, char* passwd, int port, CmdError& cmderr) = 0;
       /*! \brief
        *  Disconnects from the remote computer
        */
        virtual void            disconnect() = 0;
       /*! \brief
        *  Executes an ls command. Requires the path of the file/directory as a parameter. 
        *  Forward slash can be used to separate the driectories, on windows as well.
        *  If the path includes spaces, the path needs to be enclosed in double quotation marks. 
        */
        virtual CmdReplyFile*   execLs(char cmdargs[], CmdError& cmderr) = 0;
       /*! \brief
        *  Remove the file or directory specified in the first argument 
        */
        virtual int             execRm(char cmdargs[], CmdError& cmderr) = 0;
       /*! \brief
        *  Returns the MD5 sum of the specified file
        */
        virtual CmdReplyMd5sum* execMd5sum(char cmdargs[], CmdError& cmderr) = 0;
       /*! \brief
        *  Copies a file. Requires two arguments, source and destination
        */
        virtual int             execCp(char src[], char dest[], CmdError& cmderr) = 0;
       /*! \brief
        *  Retrieves the file system disk usage information 
        */
       virtual CmdReplyMounts* execDf(CmdError& cmderr) = 0;
       /*! \brief
        *  Retrieves the disk usage information of a file/directory
        */
       virtual  CmdReplySize*   execDu(char cmdargs[], CmdError& cmderr) = 0;
       /*! \brief
        *  Retrives the system information of the remote information. This information is same as that returned by the connect method.
        */
       virtual  CmdReplyInfo*   execInfo(CmdError& cmderr) = 0;
       /*! \brief
        *  Creates a directory with the specified name
        */
       virtual  int             execMkdir(char cmdargs[], CmdError& cmderr) = 0;
       /*! \brief
        *  Moves a file. Requires two arguments, source and destination
        */
       virtual  int             execMv(char src[], char dest[], CmdError& cmderr) = 0;

       //virtual void             Destroy() = 0;

        /*! \brief
        *  Cannot directly create a CommandClient. Use CreateInstance() method instead
        */
        CommandClient();

        /*! \brief
        *  Cannot call the destructor. Use the Destroy() method instead
        */
        virtual ~CommandClient();
    };
}
}
#endif  /* __COMMANDCLIENT_H__ */
